@@ -74,6 +74,7 @@ from prompt_toolkit.layout.processors import (
 from prompt_toolkit.lexers import DynamicLexer, Lexer
 from prompt_toolkit.mouse_events import MouseEvent, MouseEventType
 from prompt_toolkit.utils import get_cwidth
+from prompt_toolkit.validation import DynamicValidator, Validator
 
 from .toolbars import SearchToolbar
 
@@ -139,6 +140,8 @@ class TextArea:
     :param focus_on_click: When `True`, focus after mouse click.
     :param input_processors: `None` or a list of
         :class:`~prompt_toolkit.layout.Processor` objects.
+    :param validator: `None` or a :class:`~prompt_toolkit.validation.Validator`
+        object.
 
     Window attributes:
 
@@ -173,6 +176,7 @@ class TextArea:
         auto_suggest: Optional[AutoSuggest] = None,
         completer: Optional[Completer] = None,
         complete_while_typing: FilterOrBool = True,
+        validator: Optional[Validator] = None,
         accept_handler: Optional[BufferAcceptHandler] = None,
         history: Optional[History] = None,
         focusable: FilterOrBool = True,
@@ -208,6 +212,7 @@ class TextArea:
         self.auto_suggest = auto_suggest
         self.read_only = read_only
         self.wrap_lines = wrap_lines
+        self.validator = validator
 
         self.buffer = Buffer(
             document=Document(text, 0),
@@ -217,6 +222,7 @@ class TextArea:
             complete_while_typing=Condition(
                 lambda: is_true(self.complete_while_typing)
             ),
+            validator=DynamicValidator(lambda: self.validator),
             auto_suggest=DynamicAutoSuggest(lambda: self.auto_suggest),
             accept_handler=accept_handler,
             history=history,
@@ -257,6 +263,10 @@ class TextArea:
 
         style = "class:text-area " + style
 
+        # If no height was given, guarantee height of at least 1.
+        if height is None:
+            height = D(min=1)
+
         self.window = Window(
             height=height,
             width=width,
@@ -346,6 +356,7 @@ class Label:
         self.window = Window(
             content=self.formatted_text_control,
             width=get_width,
+            height=D(min=1),
             style="class:label " + style,
             dont_extend_height=dont_extend_height,
             dont_extend_width=dont_extend_width,
@@ -367,10 +378,17 @@ class Button:
     """
 
     def __init__(
-        self, text: str, handler: Optional[Callable[[], None]] = None, width: int = 12
+        self,
+        text: str,
+        handler: Optional[Callable[[], None]] = None,
+        width: int = 12,
+        left_symbol: str = "<",
+        right_symbol: str = ">",
     ) -> None:
 
         self.text = text
+        self.left_symbol = left_symbol
+        self.right_symbol = right_symbol
         self.handler = handler
         self.width = width
         self.control = FormattedTextControl(
@@ -385,18 +403,27 @@ class Button:
             else:
                 return "class:button"
 
+        # Note: `dont_extend_width` is False, because we want to allow buttons
+        #       to take more space if the parent container provides more space.
+        #       Otherwise, we will also truncate the text.
+        #       Probably we need a better way here to adjust to width of the
+        #       button to the text.
+
         self.window = Window(
             self.control,
             align=WindowAlign.CENTER,
             height=1,
             width=width,
             style=get_style,
-            dont_extend_width=True,
+            dont_extend_width=False,
             dont_extend_height=True,
         )
 
     def _get_text_fragments(self) -> StyleAndTextTuples:
-        text = ("{:^%s}" % (self.width - 2)).format(self.text)
+        width = self.width - (
+            get_cwidth(self.left_symbol) + get_cwidth(self.right_symbol)
+        )
+        text = ("{{:^{}}}".format(width)).format(self.text)
 
         def handler(mouse_event: MouseEvent) -> None:
             if (
@@ -406,10 +433,10 @@ class Button:
                 self.handler()
 
         return [
-            ("class:button.arrow", "<", handler),
+            ("class:button.arrow", self.left_symbol, handler),
             ("[SetCursorPosition]", ""),
             ("class:button.text", text, handler),
-            ("class:button.arrow", ">", handler),
+            ("class:button.arrow", self.right_symbol, handler),
         ]
 
     def _get_key_bindings(self) -> KeyBindings:
@@ -508,7 +535,9 @@ class Frame:
                         fill(width=1, height=1, char=Border.BOTTOM_LEFT),
                         fill(char=Border.HORIZONTAL),
                         fill(width=1, height=1, char=Border.BOTTOM_RIGHT),
-                    ]
+                    ],
+                    # specifying height here will increase the rendering speed.
+                    height=1,
                 ),
             ],
             width=width,
